@@ -137,22 +137,6 @@ sub interpolate_class{
     return( $class, @traits );
 }
 
-sub _coerce_and_verify {
-    #my($self, $value, $instance) = @_;
-    my($self, $value) = @_;
-
-    my $type_constraint = $self->{type_constraint};
-    return $value if !defined $type_constraint;
-
-    if ($self->should_coerce && $type_constraint->has_coercion) {
-        $value = $type_constraint->coerce($value);
-    }
-
-    $self->verify_against_type_constraint($value);
-
-    return $value;
-}
-
 sub verify_against_type_constraint {
     my ($self, $value) = @_;
 
@@ -277,7 +261,7 @@ sub install_accessors{
 
     # install delegation
     if(exists $attribute->{handles}){
-        my %handles = $attribute->_canonicalize_handles($attribute->{handles});
+        my %handles = $attribute->_canonicalize_handles();
 
         while(my($handle, $method_to_call) = each %handles){
             if($metaclass->has_method($handle)) {
@@ -300,35 +284,47 @@ sub delegation_metaclass() { ## no critic
 }
 
 sub _canonicalize_handles {
-    my($self, $handles) = @_;
+    my($self) = @_;
+    my $handles = $self->{handles};
 
-    if (ref($handles) eq 'HASH') {
+    my $handle_type = ref $handles;
+    if ($handle_type eq 'HASH') {
         return %$handles;
     }
-    elsif (ref($handles) eq 'ARRAY') {
+    elsif ($handle_type eq 'ARRAY') {
         return map { $_ => $_ } @$handles;
     }
-    elsif ( ref($handles) eq 'CODE' ) {
-        my $class_or_role = ( $self->{isa} || $self->{does} )
-            || $self->throw_error( "Cannot find delegate metaclass for attribute " . $self->name );
-        return $handles->( $self, Mouse::Meta::Class->initialize("$class_or_role"));
-    }
-    elsif (ref($handles) eq 'Regexp') {
-        my $class_or_role = ($self->{isa} || $self->{does})
-            || $self->throw_error("Cannot delegate methods based on a Regexp without a type constraint (isa)");
-
-        my $meta = Mouse::Meta::Class->initialize("$class_or_role"); # "" for stringify
+    elsif ($handle_type eq 'Regexp') {
+        my $meta = $self->_find_delegate_metaclass();
         return map  { $_ => $_ }
                grep { !Mouse::Object->can($_) && $_ =~ $handles }
                    Mouse::Util::is_a_metarole($meta)
                         ? $meta->get_method_list
                         : $meta->get_all_method_names;
     }
+    elsif ($handle_type eq 'CODE') {
+        return $handles->( $self, $self->_find_delegate_metaclass() );
+    }
     else {
         $self->throw_error("Unable to canonicalize the 'handles' option with $handles");
     }
 }
 
+sub _find_delegate_metaclass {
+    my($self) = @_;
+    my $meta;
+    if($self->{isa}) {
+        $meta = Mouse::Meta::Class->initialize("$self->{isa}");
+    }
+    elsif($self->{does}) {
+        $meta = Mouse::Util::get_metaclass_by_name("$self->{does}");
+    }
+    defined($meta) or $self->throw_error(
+        "Cannot find delegate metaclass for attribute " . $self->name);
+    return $meta;
+}
+
+
 sub _make_delegation_method {
     my($self, $handle, $method_to_call) = @_;
     return Mouse::Util::load_class($self->delegation_metaclass)
@@ -344,7 +340,7 @@ Mouse::Meta::Attribute - The Mouse attribute metaclass
 
 =head1 VERSION
 
-This document describes Mouse version 0.71
+This document describes Mouse version 0.72
 
 =head1 DESCRIPTION
 
