@@ -238,6 +238,12 @@ sub add_attribute {
         Carp::carp(qq{Attribute ($name) of class }.$self->name
             .qq{ has no associated methods (did you mean to provide an "is" argument?)});
     }
+
+    if(!Mouse::Util::MOUSE_XS) {
+        # in Mouse::PurePerl, attribute initialization code is cached, so it
+        # must be clear here. See _initialize_object() in Mouse::PurePerl.
+        delete $self->{_initialize_object};
+    }
     return $attr;
 }
 
@@ -445,7 +451,7 @@ Mouse::Meta::Class - The Mouse class metaclass
 
 =head1 VERSION
 
-This document describes Mouse version 0.71
+This document describes Mouse version 0.72
 
 =head1 DESCRIPTION
 
